import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { formatWaktu } from '@/components/Form/FiltersType/reference'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { exportToWord, exportDetailToWord } from './doc/Anomali_LAPPGP_LPP'

const reportName = 'Anomali Penanganan Pengaduan Gangguan Petugas'
const fontSize = 5
const detailFontSize = 3

const formatData = (rawData: any) => {
  const formattedData: any = []

  rawData.forEach((item: any) => {
    formattedData.push([
      item.nama_unit,
      item.nama_petugas,
      formatNumber(item.wo_total),
      formatNumber(item.wo_pln_mobile),
      formatNumber(item.wo_cc123),
      formatNumber(item.wo_pln_mobile + item.wo_cc123),
      formatPercentage(item.persen_anomali_pln_mobile),
      formatPercentage(item.persen_anomali_cc123),
      formatPercentage(item.persen_anomali_pln_mobile + item.persen_anomali_cc123)
    ])
  })

  return formattedData
}

const formatMetaData = (reportMeta: any) => {
  const periode = reportMeta.periode ? reportMeta.periode.split(' s/d ') : ''

  let dateFromFormat = ''
  let dateToFormat = ''
  let dayTo = ''

  if (periode != '') {
    const dateFrom = new Date(periode[0].split('-').reverse().join('-'))
    const dateTo = new Date(periode[1].split('-').reverse().join('-'))

    dateFromFormat = `${dateFrom.getDate()}-${dateFrom.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateFrom.getFullYear()}`

    dateToFormat = `${dateTo.getDate()}-${dateTo.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateTo.getFullYear()}`

    dayTo = dateTo.toLocaleString('id-ID', { weekday: 'long' })
  }

  return { dateFromFormat, dateToFormat, dayTo }
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const meta = formatMetaData(reportMeta)
  const data = formatData(rawData)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'UNIT LAYANAN PELANGGAN',
        ':',
        reportMeta.ulp
          ? reportMeta.ulp.name.toUpperCase()
          : 'Semua Unit Layanan Pelanggan'.toUpperCase()
      ],
      ['JENIS LAPORAN', ':', reportMeta.jenisLaporan.name.toUpperCase()]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10
  })

  autoTable(doc, {
    head: [
      [`${reportName}`.toUpperCase()],
      [`PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 25
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'Nama Unit',
          rowSpan: 3
        },
        {
          content: 'Nama Petugas',
          rowSpan: 3
        },
        'Total WO',
        {
          content: 'Total Pengaduan Yang Diselesaikan Secara Anomali',
          colSpan: 3
        },
        {
          content: '% Pengaduan Diselesaikan Secara Anomali',
          colSpan: 3
        }
      ],
      [
        '(PLN Mobile, CC123, DLL)',
        'PLN Mobile',
        'CC 123',
        'Total',
        'PLN Mobile',
        'CC 123',
        'Total'
      ],
      ['a', 'b', 'c', 'd=b+c', 'e=b/a', 'f=c/a', 'g=e+f']
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'TOTAL') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 35
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const createDetailTableField = (reportMeta: any, rawData: any) => {
  const field = ['No', 'UIW/D', 'UP3', 'Rayon', 'Nama Petugas']

  if (reportMeta.jenisLaporan.name === 'Laporan Berulang Unit') {
    field.push(
      ...[
        'No Laporan',
        'No Laporan Lapor Ulang',
        'No Laporan Referensi',
        'Status Induk',
        'Referensi Marking Induk'
      ]
    )
  }

  if (reportMeta.jenisLaporan.name === 'Laporan Rating Negatif') {
    field.push(...['Rating', 'Feedback', 'Ulasan'])
  }

  field.push(
    ...[
      'Tgl/Jam Lapor',
      'Tgl/Jam Datang',
      'Tgl/Jam Nyala',
      'Durasi Response Time',
      'Durasi Recovery Time',
      'Durasi Penugasan Regu',
      'Durasi Perjalanan Regu',
      'Jarak Closing',
      'Dispacth Oleh',
      'IDPEL/NO METER',
      'Nama Pelapor',
      'Alamat Pelapor',
      'No Telp Pelapor',
      'Keterangan Pelapor',
      'Sumber Lapor',
      'Diselesaikan Oleh',
      'Status',
      'Referensi Marking',
      'Kode Gangguan',
      'Jenis Gangguan',
      'Penyebab',
      'Tindakan'
    ]
  )

  const rows = rawData.map((item: any, i: any) => {
    const data = [
      { content: i + 1, styles: { halign: 'right' } },
      item.nama_uid,
      item.nama_up3,
      item.nama_ulp,
      item.nama_petugas
    ]

    if (reportMeta.jenisLaporan.name === 'Laporan Berulang Unit') {
      data.push(
        ...[
          item.no_laporan,
          item.no_laporan_laporulang,
          item.no_laporan,
          item.status_induk,
          item.referensi_marking_induk
        ]
      )
    }

    if (reportMeta.jenisLaporan.name === 'Laporan Rating Negatif') {
      data.push(...[item.rating, item.feedback, item.ulasan])
    }

    data.push(
      ...[
        item.waktu_lapor,
        item.waktu_response,
        item.waktu_recovery,
        item.durasi_response_time ? formatWaktu(item.durasi_response_time) : '-',
        item.durasi_recovery_time ? formatWaktu(item.durasi_recovery_time) : '-',
        item.durasi_dispatch_time ? formatWaktu(item.durasi_dispatch_time) : '-',
        item.durasi_perjalanan ? formatWaktu(item.durasi_perjalanan) : '-',
        item.distance,
        item.dispatch_by,
        item.idpel_nometer,
        item.nama_pelapor,
        item.alamat_pelapor,
        item.no_telp_pelapor,
        item.keterangan_pelapor,
        item.media,
        item.diselesaikan_oleh,
        item.status_akhir,
        item.marking,
        item.kode_gangguan,
        item.jenis_gangguan,
        item.penyebab,
        item.tindakan
      ]
    )

    return data
  })

  return { field, rows }
}

const exportDetailToPDF = (reportMeta: any, rawData: any) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const meta = formatMetaData(reportMeta)
  const { field, rows } = createDetailTableField(reportMeta, rawData)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [['PT. PLN(Persero)']],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [`Daftar Detail ${reportName}`.toUpperCase()],
      [`PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`]
    ],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 18,
    margin: 5
  })

  autoTable(doc, {
    head: [field],
    body: rows,
    styles: {
      fontSize: detailFontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }
    },
    startY: 24,
    margin: 5
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize: detailFontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  doc.save(`Laporan Detail ${reportName}.pdf`, { returnPromise: true }).then(() => {
    console.log('pdf berhasil disimpan')
  })
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const meta = formatMetaData(reportMeta)
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${
      reportMeta.uid
        ? reportMeta.uid.name.toUpperCase()
        : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${
      reportMeta.up3
        ? reportMeta.up3.name.toUpperCase()
        : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `UNIT LAYANAN PELANGGAN : ${
      reportMeta.ulp
        ? reportMeta.ulp.name.toUpperCase()
        : 'Semua Unit Layanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(worksheet, 5, 1, `JENIS LAPORAN : ${reportMeta.jenisLaporan.name.toUpperCase()}`)

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(
    worksheet,
    8,
    1,
    `PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`,
    true
  )

  worksheet.mergeCells('A1:I1')
  worksheet.mergeCells('A2:I2')
  worksheet.mergeCells('A3:I3')
  worksheet.mergeCells('A4:I4')
  worksheet.mergeCells('A5:I5')
  worksheet.mergeCells('A7:I7')
  worksheet.mergeCells('A8:I8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportDetailToXLSX = (reportMeta: any, e: any) => {
  const meta = formatMetaData(reportMeta)
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`Detail ${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(worksheet, 3, 1, `Daftar Detail ${reportName}`.toUpperCase(), true)
  setHeaderStyle(
    worksheet,
    4,
    1,
    `PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`,
    true
  )

  worksheet.mergeCells('A1:AC1')
  worksheet.mergeCells('A3:AC3')
  worksheet.mergeCells('A4:AC4')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 6, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(
        new Blob([buffer], { type: 'application/octet-stream' }),
        `Laporan Detail ${reportName}.xlsx`
      )
    })
  })

  e.cancel = true
}

const exportToDOCX = (reportMeta: any, rawData: any) => {
  const meta = formatMetaData(reportMeta)
  exportToWord(reportMeta, meta, formatData(rawData), reportName)
}

const exportDetailToDOCX = (reportMeta: any, rawData: any) => {
  const meta = formatMetaData(reportMeta)
  exportDetailToWord(meta, createDetailTableField(reportMeta, rawData), reportName)
}

export {
  exportToPDF,
  exportToXLSX,
  exportToDOCX,
  exportDetailToPDF,
  exportDetailToXLSX,
  exportDetailToDOCX
}
